/*****************************************************************************
 * vlc_vlvc_error.h: VLC's core module headers for VLVC
 *****************************************************************************
 * Copyright (C) 2005 - 2008 the VLVC team
 * $Id: vlc_vlvc_error.h
 *
 * Authors: Julien ARNAUD <arnaud_j@epitech.net>
 *          Julien CUPILLARD <cupill_j@epitech.net>
 *          Thomas DELON <delon_t@epitech.net>
 *          Arnaud DEREDEC <derede_a@epitech.net>
 *          Florian GUSTIN <gustin_f@epitech.net>
 *          Sylvestre HO TAM CHAY <ho-tam_s@epitech.net>
 *          Romain LAVOCAT <lavoca_r@epitech.net>
 *          William LIENARD <lienar_w@epitech.net>
 *          Guillaume MARTIN <martin_u@epitech.net>
 *          Florent NGUYEN <nguyen_c@epitech.net>
 *          Julien PASSERI <passer_j@epitech.net>
 *          Mathavarajan SUGUMARAN <suguma_m@epitech.net>
 *          David YU <yu_d@epitech.net>
 *
 * This software has been written in EPITECH <http://www.epitech.net>
 * EPITECH is computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud <http://www.epita.fr/~flav>,
 * Jerome Landrieu and Launiau Severin.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/

#ifndef _VLC_VLVC_ERROR_H
#define _VLC_VLVC_ERROR_H

/*****************************************************************************
 * Core module critical levels
 *****************************************************************************/
#define VLVC_CRITICAL_LEVEL_MIN          1
#define VLVC_CRITICAL_LEVEL_QUIT_LIMT    5
#define VLVC_CRITICAL_LEVEL_MAX          10
#define VLVC_CRITICAL_TITLE_LOW          N_( "Message" )
#define VLVC_CRITICAL_TITLE_HIGH         N_( "Error" )

/*****************************************************************************
 * Core module error messages
 *****************************************************************************/

#define VLVC_CORE_MODULE_ERROR_NO_ERROR                      "No error"
#define VLVC_CORE_MODULE_ERROR_NO_ERROR_INDEX                0
#define VLVC_CORE_MODULE_ERROR_BIND                          N_( "Unable to bind specified address" )
#define VLVC_CORE_MODULE_ERROR_BIND_INDEX                    (VLVC_CORE_MODULE_ERROR_NO_ERROR_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_SERVER_THREAD                 N_( "Unable to launch server thread" )
#define VLVC_CORE_MODULE_ERROR_SERVER_THREAD_INDEX           (VLVC_CORE_MODULE_ERROR_BIND_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_CONNECT                       N_( "Unable to connect to server" )
#define VLVC_CORE_MODULE_ERROR_CONNECT_INDEX                 (VLVC_CORE_MODULE_ERROR_SERVER_THREAD_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_CLIENT_THREAD                 N_( "Unable to launch client thread" )
#define VLVC_CORE_MODULE_ERROR_CLIENT_THREAD_INDEX           (VLVC_CORE_MODULE_ERROR_CONNECT_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_CONNECTION_LOST               N_( "Connection to the server lost" )
#define VLVC_CORE_MODULE_ERROR_CONNECTION_LOST_INDEX         (VLVC_CORE_MODULE_ERROR_CLIENT_THREAD_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_USER_INFO                 N_( "Unable to get user information" )
#define VLVC_CORE_MODULE_ERROR_GET_USER_INFO_INDEX           (VLVC_CORE_MODULE_ERROR_CONNECTION_LOST_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE          N_( "User index out of range" )
#define VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE_INDEX    (VLVC_CORE_MODULE_ERROR_GET_USER_INFO_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA           N_( "Unable to get conference data" )
#define VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA_INDEX     (VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND           N_( "An error occured while waiting for the welcome command" )
#define VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND_INDEX     (VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_REFUSED                       N_( "You have been refused by the server" )
#define VLVC_CORE_MODULE_ERROR_REFUSED_INDEX                 (VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE            N_( "VLVC can be launched as standalone only in server mode" )
#define VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE_INDEX      (VLVC_CORE_MODULE_ERROR_REFUSED_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_STANDALONE_GUI                N_( "Cannot use standalone mode with graphical interface" )
#define VLVC_CORE_MODULE_ERROR_STANDALONE_GUI_INDEX          (VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED              N_( "VLVC already launched" )
#define VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED_INDEX        (VLVC_CORE_MODULE_ERROR_STANDALONE_GUI_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_MULTICAST_COMMAND         N_( "An error occured while waiting for the multicast command" )
#define VLVC_CORE_MODULE_ERROR_GET_MULTICAST_COMMAND_INDEX   (VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_MULTICAST_DATA            N_( "Unable to get multicast data" )
#define VLVC_CORE_MODULE_ERROR_GET_MULTICAST_DATA_INDEX      (VLVC_CORE_MODULE_ERROR_GET_MULTICAST_COMMAND_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_INCORRECT_PASSWORD            N_( "The server is protected, Incorrect password" )
#define VLVC_CORE_MODULE_ERROR_INCORRECT_PASSWORD_INDEX      (VLVC_CORE_MODULE_ERROR_GET_MULTICAST_DATA_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_INCORRECT_VERSION             N_( "Unmatched version : Server is using version %s and your version is %s" )
#define VLVC_CORE_MODULE_ERROR_INCORRECT_VERSION_INDEX       (VLVC_CORE_MODULE_ERROR_INCORRECT_PASSWORD_INDEX + 1)
#define VLVC_CORE_MODULE_ERROR_GET_NOT_WELCOME_COMMAND       N_( "An error occured while waiting for the not welcome command" )
#define VLVC_CORE_MODULE_ERROR_GET_NOT_WELCOME_COMMAND_INDEX (VLVC_CORE_MODULE_ERROR_INCORRECT_VERSION_INDEX + 1)

/*****************************************************************************
 * Core module error values
 *****************************************************************************/
char *p_vlvc_error_messages[] = {
    VLVC_CORE_MODULE_ERROR_NO_ERROR,
    VLVC_CORE_MODULE_ERROR_BIND,
    VLVC_CORE_MODULE_ERROR_SERVER_THREAD,
    VLVC_CORE_MODULE_ERROR_CONNECT,
    VLVC_CORE_MODULE_ERROR_CLIENT_THREAD,
    VLVC_CORE_MODULE_ERROR_CONNECTION_LOST,
    VLVC_CORE_MODULE_ERROR_GET_USER_INFO,
    VLVC_CORE_MODULE_ERROR_USER_ID_OUT_OF_RANGE,
    VLVC_CORE_MODULE_ERROR_GET_CONFERENCE_DATA,
    VLVC_CORE_MODULE_ERROR_GET_WELCOME_COMMAND,
    VLVC_CORE_MODULE_ERROR_REFUSED,
    VLVC_CORE_MODULE_ERROR_INVALID_STANDALONE,
    VLVC_CORE_MODULE_ERROR_STANDALONE_GUI,
    VLVC_CORE_MODULE_ERROR_ALREADY_LAUNCHED,
    VLVC_CORE_MODULE_ERROR_GET_MULTICAST_COMMAND,
    VLVC_CORE_MODULE_ERROR_GET_MULTICAST_DATA,
    VLVC_CORE_MODULE_ERROR_INCORRECT_PASSWORD,
    VLVC_CORE_MODULE_ERROR_INCORRECT_VERSION,
    VLVC_CORE_MODULE_ERROR_GET_NOT_WELCOME_COMMAND
};

#endif /* _VLC_VLVC_ERROR_H */
