/*****************************************************************************
 * vlvc.c: Access module for VLVC
 *****************************************************************************
 * Copyright (C) 2005 - 2008 the VLVC team
 * $Id: vlvc.c
 *
 * Authors: Julien ARNAUD <arnaud_j@epitech.net>
 *          Julien CUPILLARD <cupill_j@epitech.net>
 *          Thomas DELON <delon_t@epitech.net>
 *          Arnaud DEREDEC <derede_a@epitech.net>
 *          Florian GUSTIN <gustin_f@epitech.net>
 *          Sylvestre HO TAM CHAY <ho-tam_s@epitech.net>
 *          Romain LAVOCAT <lavoca_r@epitech.net>
 *          William LIENARD <lienar_w@epitech.net>
 *          Guillaume MARTIN <martin_u@epitech.net>
 *          Florent NGUYEN <nguyen_c@epitech.net>
 *          Julien PASSERI <passer_j@epitech.net>
 *          Mathavarajan SUGUMARAN <suguma_m@epitech.net>
 *          David YU <yu_d@epitech.net
 *
 * This software has been written in EPITECH <http://www.epitech.net>
 * EPITECH is computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud <http://www.epita.fr/~flav>,
 * Jerome Landrieu and Launiau Severin.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/

#include <stdlib.h>

#include <vlc/vlc.h>
#include <vlc/input.h>

#include <vlc_vlvc.h>

#include "vlvc.h"

#include <vlc_vlvc_error.h>
#include <vlc_md5.h>

vlc_module_begin();
    set_shortname( _( VLVC_MODULE_SHORTNAME ) );
    set_description( _( VLVC_MODULE_DESCRIPTION ) );
    set_category( CAT_INPUT );
set_subcategory( SUBCAT_INPUT_ACCESS );

    add_bool( VLVC_VAR_SERVER_MODE, VLVC_VAR_SERVER_MODE_DEFAULT, NULL,
              SERVER_MODE_TEXT, SERVER_MODE_LONGTEXT, VLVC_ADVANCED_SERVER_MODE );

    add_string( VLVC_VAR_IP, VLVC_VAR_IP_DEFAULT, NULL, HOST_IP_ADDR_TEXT,
                HOST_IP_ADDR_LONGTEXT, VLVC_ADVANCED_IP );

    add_integer( VLVC_VAR_CONFERENCE_MODE, VLVC_VAR_CONFERENCE_MODE_DEFAULT,
                 NULL, CONFERENCE_MODE_TEXT, CONFERENCE_MODE_LONGTEXT,
                 VLVC_ADVANCED_CONFERENCE_MODE );
    change_integer_list( i_conference_mode_list,
                         psz_conference_mode_list_text, 0 );

    add_integer( VLVC_VAR_VIDEO_CAPTURE_SIZE, VLVC_VAR_VIDEO_CAPTURE_SIZE_DEFAULT, NULL,
                 VIDEO_CAPTURE_SIZE_TEXT, VIDEO_CAPTURE_SIZE_LONGTEXT, VLVC_ADVANCED_VIDEO_CAPTURE_SIZE );
    change_integer_list( i_video_capture_size_list, psz_video_capture_size_list_text, 0 );

    add_integer( VLVC_VAR_VIDEO_SCALE, VLVC_VAR_VIDEO_SCALE_DEFAULT, NULL,
                 VIDEO_SCALE_TEXT, VIDEO_SCALE_LONGTEXT, VLVC_ADVANCED_VIDEO_SCALE );
    change_integer_list( i_video_scale_list, psz_video_scale_list, 0 );

    add_integer( VLVC_VAR_VCODEC, VLVC_VAR_VCODEC_DEFAULT, NULL,
                 VCODEC_TEXT, VCODEC_LONGTEXT, VLVC_ADVANCED_VCODEC );
    change_integer_list( i_vcodec_list, psz_vcodec_list, 0 );

    add_integer( VLVC_VAR_VBITRATE, VLVC_VAR_VBITRATE_DEFAULT, NULL,
                 VBITRATE_TEXT, VBITRATE_LONGTEXT, VLVC_ADVANCED_VBITRATE );
    change_integer_list( i_vbitrate_list, psz_vbitrate_list_text, 0 );

    add_integer( VLVC_VAR_ACODEC, VLVC_VAR_ACODEC_DEFAULT, NULL,
                 ACODEC_TEXT, ACODEC_LONGTEXT, VLVC_ADVANCED_ACODEC );
    change_integer_list( i_acodec_list, psz_acodec_list, 0 );

    add_integer( VLVC_VAR_ABITRATE, VLVC_VAR_ABITRATE_DEFAULT, NULL,
                 ABITRATE_TEXT, ABITRATE_LONGTEXT, VLVC_ADVANCED_ABITRATE );
    change_integer_list( i_abitrate_list, psz_abitrate_list_text, 0 );

    add_bool( VLVC_VAR_FORCE_CODECS, VLVC_VAR_FORCE_CODECS_DEFAULT, NULL,
              FORCE_CODECS_TEXT, FORCE_CODECS_LONGTEXT, VLVC_ADVANCED_FORCE_CODECS );

    add_string( VLVC_VAR_PASSWORD, VLVC_VAR_PASSWORD_DEFAULT, NULL,
                PASSWORD_TEXT, PASSWORD_LONGTEXT, VLVC_ADVANCED_PASSWORD );

    add_string( VLVC_VAR_USERNAME, VLVC_VAR_USERNAME_DEFAULT, NULL,
                HOST_USERNAME_TEXT, HOST_USERNAME_LONGTEXT, VLVC_ADVANCED_USERNAME );

    add_string( VLVC_VAR_DESCRIPTION, VLVC_VAR_DESCRIPTION_DEFAULT, NULL,
                HOST_DESCRIPTION_TEXT, HOST_DESCRIPTION_LONGTEXT, VLVC_ADVANCED_DESCRIPTION );

    add_integer( VLVC_VAR_NETWORK_PROTOCOL, VLVC_VAR_NETWORK_PROTOCOL_DEFAULT,
                 NULL, NETWORK_PROTOCOL_TEXT, NETWORK_PROTOCOL_LONGTEXT,
                 VLVC_ADVANCED_NETWORK_PROTOCOL );
    change_integer_list( i_network_protocol_list,
                         psz_network_protocol_list, 0 );

    add_integer( VLVC_VAR_BUFFER_LEVEL, VLVC_VAR_BUFFER_LEVEL_DEFAULT, NULL,
                 BUFFER_LEVEL_TEXT, BUFFER_LEVEL_LONGTEXT, VLVC_ADVANCED_BUFFER_LEVEL );

    change_integer_list( i_buffer_level_list,
                         psz_buffer_level_list_text, 0 );

    add_string( VLVC_VAR_INPUT_MRL, VLVC_VAR_INPUT_MRL_DEFAULT, NULL,
                INPUT_MRL_TEXT, INPUT_MRL_LONGTEXT, VLVC_ADVANCED_INPUT_MRL );

    add_string( VLVC_VAR_VIDEO_DEVICE, VLVC_VAR_VIDEO_DEVICE_DEFAULT,
                NULL, VDEV_TEXT, VDEV_LONGTEXT, VLVC_ADVANCED_VIDEO_DEVICE );
    add_string( VLVC_VAR_AUDIO_DEVICE, VLVC_VAR_AUDIO_DEVICE_DEFAULT,
                NULL, ADEV_TEXT, ADEV_LONGTEXT, VLVC_ADVANCED_AUDIO_DEVICE );

    add_integer( VLVC_VAR_PORT, VLVC_VAR_PORT_DEFAULT, NULL, HOST_PORT_TEXT,
                 HOST_PORT_LONGTEXT, VLVC_ADVANCED_PORT );

    add_integer( VLVC_VAR_FIRST_CLIENT_PORT, VLVC_VAR_FIRST_CLIENT_PORT_DEFAULT,
                 NULL, FIRST_CLIENT_PORT_TEXT, FIRST_CLIENT_PORT_LONGTEXT,
                 VLVC_ADVANCED_FIRST_CLIENT_PORT);

    add_bool( VLVC_VAR_MULTICAST_MODE, VLVC_VAR_MULTICAST_MODE_DEFAULT, NULL,
              MULTICAST_MODE_TEXT, MULTICAST_MODE_LONGTEXT, VLVC_ADVANCED_MULTICAST_MODE );

    add_bool( VLVC_VAR_FIX_OVERLAY, VLVC_VAR_FIX_OVERLAY_DEFAULT, NULL,
              FIX_OVERLAY_TEXT, FIX_OVERLAY_LONGTEXT, VLVC_ADVANCED_FIX_OVERLAY );

    add_bool( VLVC_VAR_STANDALONE_MODE, VLVC_VAR_STANDALONE_MODE_DEFAULT, NULL,
              STANDALONE_MODE_TEXT, STANDALONE_MODE_LONGTEXT, VLVC_ADVANCED_STANDALONE_MODE );

    set_capability( VLVC_MODULE_CAPABILITY, 0 );
    add_shortcut( VLVC_MODULE_SHORTCUT );
    set_callbacks( Open, Close );
vlc_module_end();

/*****************************************************************************
 * Local prototypes
 *****************************************************************************/
static int Control( access_t *, int, va_list );
static int Block( access_t *p_access);
void InitAccess( access_t *p_access );
int GetClientServer( access_t *p_access, vlvc_param_t *p_sys );
void GetConferenceMode( access_t *p_access, vlvc_param_t *p_sys );
void GetAudioVideo( access_t *p_access, vlvc_param_t *p_sys );
void GetInputMRL( access_t *p_access, vlvc_param_t *p_sys );
void GetUsernameDescription( access_t *p_access, vlvc_param_t *p_sys );
int GetAudioVideoDevices( access_t *p_access, vlvc_param_t *p_sys );
void GetNetworkProtocol( access_t *p_access, vlvc_param_t *p_sys);
void GetCachingValues( access_t *p_access, vlvc_param_t *p_sys );
int InitVideoSize( access_t *p_access, vlvc_param_t *p_sys );
int GetOptions( access_t *p_access, vlvc_param_t *p_sys );
void GetPassword( access_t *p_access, vlvc_param_t *p_sys );
void SetVersion( vlvc_param_t *p_sys );
int GetMulticastMode( access_t *p_access, vlvc_param_t *p_sys );
void LaunchCoreModule( vlc_object_t *p_this, vlvc_param_t *p_sys );

/*****************************************************************************
 * Name: Open
 *
 * Parameters:
 *             vlc_object_t * p_this: a pointer to the common object structure
 *
 * Returns:
 *             int: VLC return code
 *                  VLC_SUCCESS in case of success
 *                  VLC_ENOMEM if not enough memory
 *
 * Description:
 *             Loads the conference settings,
 *             then launches the core module for the conference to begin
 *****************************************************************************/

static int Open( vlc_object_t *p_this )
{
    access_t     *p_access = (access_t*)p_this;
    vlvc_param_t *p_sys;
    int          i_ret;
    int          b_server;

    if( ( p_sys = malloc( sizeof( *p_sys ) ) ) == NULL )
        return( VLC_ENOMEM );

    p_access->p_sys = (access_sys_t *)p_sys;
    memset( p_sys, 0, sizeof( *p_sys ) );

    p_sys->i_conference_mode = VLVC_CONFERENCE_MODE_INVALID;

    InitAccess( p_access );
    b_server = GetClientServer( p_access, p_sys );

    if( b_server == VLC_TRUE )
        GetConferenceMode( p_access, p_sys );
    GetAudioVideo( p_access, p_sys );
    GetInputMRL( p_access, p_sys );
    GetUsernameDescription( p_access, p_sys );
    SetVersion( p_sys );
    GetPassword( p_access, p_sys );
    GetNetworkProtocol( p_access, p_sys );
    GetCachingValues( p_access, p_sys );
    GetFixOverlay( p_access, p_sys );
    GetMulticastMode( p_access, p_sys );

    if( ( i_ret = GetOptions( p_access, p_sys ) ) != VLC_SUCCESS )
        return( i_ret );

    LaunchCoreModule( p_this, p_sys );
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: Close
 *
 * Parameters:
 *             vlc_object_t * p_this: a pointer to the common object structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             free unused data structures
 *****************************************************************************/
static void Close( vlc_object_t *p_this )
{
    vlvc_param_t *p_sys;
    vlvc_t *p_vlvc;
    int i_cpt;

    p_sys = ( vlvc_param_t * )( ( access_t* )p_this )->p_sys;
    p_vlvc = vlvc_New( p_this );
    p_vlvc->p_parameters = p_sys;
    vlvc_Stop( p_vlvc );
    vlvc_Delete( p_vlvc );
    if( p_sys->b_standalone == VLC_TRUE )
        vlvc_Delete( p_vlvc );
    for(i_cpt = 0; p_sys->psz_input_options[i_cpt] != NULL; i_cpt++)
        free( p_sys->psz_input_options[i_cpt] );
    free( p_sys );
}

/*****************************************************************************
 * Name: Block
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *
 * Returns:
 *             int: VLC return code
 *
 * Description:
 *             Needed to be an access module, but useless in our case.
 *****************************************************************************/
static int Block( access_t *p_access)
{
    return( VLC_FALSE );
}

/*****************************************************************************
 * Name: Control
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             int i_query: identifies the query type on the access module
 *             va_list args: arguments, unused in our case
 *
 *
 * Returns:
 *             int: depending on the i_query parameter:
 *                  VLC_TRUE or VLC_FALSE if VLC is asking for capabilities
 *                  A value if VLC is asking for a value
 *
 * Description:
 *             Called by VLC to know the module capabilities,
 *             and various values.
 *****************************************************************************/
static int Control( access_t *p_access, int i_query, va_list args )
{
    switch( i_query )
    {

        case ACCESS_CAN_SEEK:
        case ACCESS_CAN_FASTSEEK:
        case ACCESS_CAN_PAUSE:
        case ACCESS_CAN_CONTROL_PACE:
            return( VLC_FALSE );
        case ACCESS_GET_MTU:
            return( 0 );
            break;

        case ACCESS_GET_PTS_DELAY:
            return( 0 );
            break;

        default:
            msg_Warn( p_access, "vlvc control error" );
            return( VLC_EGENERIC );

    }
    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: CopyAndAddBackslashes
 *
 * Parameters:
 *             char *psz_src: the source string
 *             char *psz_dest: the destination string
 *             int i_dest_size: the size of the destination string
 *
 * Returns:
 *             void
 *
 * Description:
 *             Copies the source in the destination, and adds backslashes
 *             before spaces.
 *****************************************************************************/
static void CopyAndAddBackslashes( char *psz_dest, char *psz_src, int i_dest_size )
{
    char *psz_dest_end = psz_dest + i_dest_size - 1;

    memset( psz_dest, 0, i_dest_size );
    while( *psz_src && psz_dest != psz_dest_end )
    {
        if( *psz_src == ' ' )
        {
            *psz_dest++ = '\\';
            if( psz_dest == psz_dest_end )
            {
                /* If the backslash just added is the last
                   character of the string */
                *( psz_dest - 1 ) = '\0';
                break;
            }

        }
        *psz_dest++ = *psz_src++;
    }
}

/*****************************************************************************
 * Name: GetNumberOfSpaces
 *
 * Parameters:
 *             char *psz_src: the source string
 *
 * Returns:
 *             int
 *
 * Description:
 *             Counts the number of spaces in the source string.
 *****************************************************************************/
static int GetNumberOfSpaces( char *psz_src )
{
    int i_nb_spaces = 0;

    while( *psz_src )
        if( *psz_src++ == ' ' )
            i_nb_spaces++;
    return( i_nb_spaces );
}

/*****************************************************************************
 * Name: InitAccess
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the module data
 *****************************************************************************/
void InitAccess( access_t *p_access )
{
    p_access->pf_seek = NULL;
    p_access->pf_block = ( block_t* ( * )( access_t* ) )Block;
    p_access->pf_read = NULL;
    p_access->pf_control = Control;
    p_access->info.i_update = 0;
    p_access->info.i_size = 0;
    p_access->info.i_pos = 0;
    p_access->info.b_eof = VLC_FALSE;
    p_access->info.b_prebuffered = VLC_FALSE;
    p_access->info.i_title = 0;
    p_access->info.i_seekpoint = 0;
}

/*****************************************************************************
 * Name: GetClientServer
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             int: VLC_TRUE if the module is launched in server mode
 *                  VLC_FALSE if the module is launched in client mode
 *
 * Description:
 *             Initializes the network-depending data selected by the user,
 *             such as the server address or the bind address,
 *             the ports to use, the standalone mode...
 *****************************************************************************/
int GetClientServer( access_t *p_access, vlvc_param_t *p_sys )
{
    char         *psz_ip;
    vlc_value_t  val;
    int          b_server;

    var_Create( p_access, VLVC_VAR_SERVER_MODE, VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_SERVER_MODE, &val );
    b_server = val.b_bool;

    if( b_server == VLC_TRUE )
        p_sys->i_operating_mode = VLVC_SERVER_MODE;
    else
        p_sys->i_operating_mode = VLVC_CLIENT_MODE;

    var_Create( p_access, VLVC_VAR_STANDALONE_MODE, VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_STANDALONE_MODE, &val );
    p_sys->b_standalone = val.b_bool;

    var_Create( p_access, VLVC_VAR_PORT, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_PORT, &val );
    p_sys->i_server_port = val.i_int;
    p_sys->i_bind_port = val.i_int;

    var_Create( p_access, VLVC_VAR_FIRST_CLIENT_PORT,
                VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_FIRST_CLIENT_PORT, &val );
    p_sys->i_first_client_port = val.i_int;

    psz_ip = var_CreateGetString( p_access, VLVC_VAR_IP );

    if( b_server == VLC_TRUE )
    {
        if( strcmp( psz_ip, "" ) )
        {
            snprintf( p_sys->psz_bind_address, sizeof( p_sys->psz_bind_address ),
                     psz_ip );
            snprintf( p_sys->psz_server_address,
                     sizeof( p_sys->psz_server_address ), psz_ip );

        }
        else
        {
            strcpy( p_sys->psz_bind_address, VLVC_DEFAULT_BIND_ADDRESS );
            strcpy( p_sys->psz_server_address, VLVC_DEFAULT_LOOPBACK_ADDRESS );
        }
    }
    else
    {
        snprintf( p_sys->psz_server_address,
                 sizeof( p_sys->psz_server_address ), psz_ip );
    }
    free( psz_ip );
    return( b_server );
}

/*****************************************************************************
 * Name: GetConferenceMode
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the conference mode selected by the user.
 *****************************************************************************/
void GetConferenceMode( access_t *p_access, vlvc_param_t *p_sys  )
{
    vlc_value_t  val;

    var_Create( p_access, VLVC_VAR_CONFERENCE_MODE,
                VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_CONFERENCE_MODE, &val );
    p_sys->i_conference_mode = val.i_int;
}

/*****************************************************************************
 * Name: GetAudioVideo
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the audio/video parameters (codec/bitrate) selected
 *             by the user.
 *****************************************************************************/
void GetAudioVideo( access_t *p_access, vlvc_param_t *p_sys )
{
    int          i_vcodec;
    int          i_acodec;
    int          i_vbitrate;
    int          i_abitrate;
    vlc_value_t  val;

    var_Create( p_access, VLVC_VAR_VCODEC, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_VCODEC, &val );
    i_vcodec = val.i_int;

    var_Create( p_access, VLVC_VAR_ACODEC, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_ACODEC, &val );
    i_acodec = val.i_int;

    var_Create( p_access, VLVC_VAR_VBITRATE, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_VBITRATE, &val );
    i_vbitrate = val.i_int;

    var_Create( p_access, VLVC_VAR_ABITRATE, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_ABITRATE, &val );
    i_abitrate = val.i_int;

    snprintf( p_sys->psz_video_codec, sizeof( p_sys->psz_video_codec ),
              psz_vcodec_list[i_vcodec] );
    snprintf( p_sys->psz_audio_codec, sizeof( p_sys->psz_audio_codec ),
              psz_acodec_list[i_acodec] );
    p_sys->i_video_bitrate = psz_vbitrate_list[i_vbitrate];
    p_sys->i_audio_bitrate = psz_abitrate_list[i_abitrate];

    var_Create( p_access, VLVC_VAR_VIDEO_SCALE, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_VIDEO_SCALE, &val );
    snprintf( p_sys->psz_video_scale, sizeof( p_sys->psz_video_scale ),
              psz_video_scale_list[val.i_int] );

    var_Create( p_access, VLVC_VAR_FORCE_CODECS, VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_FORCE_CODECS, &val );
    p_sys->b_force_codecs = val.b_bool;
}

/*****************************************************************************
 * Name: GetInputMRL
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the input MRL
 *****************************************************************************/
void GetInputMRL( access_t *p_access, vlvc_param_t *p_sys )
{
    char *psz_mrl;

    psz_mrl = var_CreateGetString( p_access, VLVC_VAR_INPUT_MRL );
    CopyAndAddBackslashes( p_sys->psz_input_mrl, psz_mrl,
                           sizeof( p_sys->psz_input_mrl ) );
    free( psz_mrl );
}

/*****************************************************************************
 * Name: GetUsernameDescription
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the username and description, provided by the user
 *****************************************************************************/
void GetUsernameDescription( access_t *p_access, vlvc_param_t *p_sys )
{
    char *psz_tmp;

    psz_tmp = var_CreateGetString( p_access, VLVC_VAR_USERNAME );
    strncpy( p_sys->psz_username, psz_tmp, VLVC_BUFFER_SIZE( p_sys->psz_username ) );
    free( psz_tmp );
    psz_tmp = var_CreateGetString( p_access, VLVC_VAR_DESCRIPTION );
    strncpy( p_sys->psz_desc, psz_tmp, VLVC_BUFFER_SIZE( p_sys->psz_desc ) );
    free( psz_tmp );
}

/*****************************************************************************
 * Name: GetPassword
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the password provided by the user
 *****************************************************************************/
void GetPassword( access_t *p_access, vlvc_param_t *p_sys )
{
    char *psz_tmp;
    struct md5_s p_md5;
    int i;

    InitMD5( &p_md5 );
    psz_tmp = var_CreateGetString( p_access, VLVC_VAR_PASSWORD );
    if( strcmp( psz_tmp, "" ) )
    {
        AddMD5( &p_md5, psz_tmp, strlen( psz_tmp ) );
        EndMD5( &p_md5 );
        memset( p_sys->psz_password,'\0', VLVC_MD5 );
        for( i = 0; i < 4; i++ )
            sprintf( p_sys->psz_password, "%s%02x", p_sys->psz_password,
                     p_md5.p_digest[i] );
        free( psz_tmp );
    }
    else
    {
        memset( p_sys->psz_password,'\0', VLVC_MD5 );
    }
}

/*****************************************************************************
 * Name: SetVersion
 *
 * Parameters:
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the version string of VLVC
 *****************************************************************************/
void SetVersion( vlvc_param_t *p_sys )
{
    strncpy( p_sys->psz_version, VLVC_VERSION, VLVC_BUFFER_SIZE( p_sys->psz_version ) );
}

/*****************************************************************************
 * Name: GetNetworkProtocol
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the network protocol
 *****************************************************************************/
void GetNetworkProtocol( access_t *p_access, vlvc_param_t *p_sys )
{
    vlc_value_t  val;

    var_Create( p_access, VLVC_VAR_NETWORK_PROTOCOL, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_NETWORK_PROTOCOL, &val );
    strcpy( p_sys->psz_network_protocol, psz_network_protocol_list[val.i_int] );
}

/*****************************************************************************
 * Name: GetCachingValues
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Initializes the caching values selected by the user
 *****************************************************************************/
void GetCachingValues( access_t *p_access, vlvc_param_t *p_sys )
{
    vlc_value_t  val;

    var_Create( p_access, VLVC_VAR_BUFFER_LEVEL, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_BUFFER_LEVEL, &val );
    memcpy( &p_sys->caching_values, &caching_values[val.i_int],
            sizeof( p_sys->caching_values ) );
}

/*****************************************************************************
 * Name: GetAudioVideoDevices
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             int: VLC return code:
 *                  VLC_SUCCESS in case of success
 *                  VLC_ENOMEM if not enough memory
 *
 * Description:
 *             Initializes the audio and video devices selected by the user
 *****************************************************************************/
int GetAudioVideoDevices( access_t *p_access, vlvc_param_t *p_sys )
{
    char *psz_tmp;
    int  i_tmp;
    char *psz_with_backslashes;

    psz_tmp = var_CreateGetString( p_access, VLVC_VAR_VIDEO_DEVICE );

    i_tmp = strlen( psz_tmp ) + GetNumberOfSpaces( psz_tmp ) + 1;
    psz_with_backslashes = malloc( i_tmp );

    if( !psz_with_backslashes)
    {
        free( psz_tmp );
        return( VLC_ENOMEM );
    }

    CopyAndAddBackslashes( psz_with_backslashes, psz_tmp, i_tmp );

    i_tmp = strlen( psz_with_backslashes ) + strlen( VLVC_OPT_VDEV_BASE );
    p_sys->psz_input_options[VLVC_IDX_OPT_VDEV] =
        malloc( i_tmp  * sizeof( *p_sys->psz_input_options[VLVC_IDX_OPT_VDEV] ) );
    if( !p_sys->psz_input_options[VLVC_IDX_OPT_VDEV] )
    {
        free( p_sys->psz_input_options );
        free( psz_tmp );
        free( psz_with_backslashes );
        return( VLC_ENOMEM );
    }

    sprintf( p_sys->psz_input_options[VLVC_IDX_OPT_VDEV],
             VLVC_OPT_VDEV_BASE, psz_with_backslashes );
    free( psz_tmp );
    free( psz_with_backslashes );

    psz_tmp = var_CreateGetString( p_access, VLVC_VAR_AUDIO_DEVICE );

    i_tmp = strlen( psz_tmp ) + GetNumberOfSpaces( psz_tmp ) + 1;

    psz_with_backslashes = malloc( i_tmp );

    if( !psz_with_backslashes )
    {
        free( psz_tmp );
        return( VLC_ENOMEM );
    }

    CopyAndAddBackslashes( psz_with_backslashes, psz_tmp, i_tmp );

    i_tmp = strlen( psz_with_backslashes ) + strlen( VLVC_OPT_ADEV_BASE ) ;
    p_sys->psz_input_options[VLVC_IDX_OPT_ADEV] =
        malloc( i_tmp  * sizeof( *p_sys->psz_input_options[VLVC_IDX_OPT_ADEV] ) );
    if( !p_sys->psz_input_options[VLVC_IDX_OPT_ADEV] )
    {
        free( p_sys->psz_input_options[VLVC_IDX_OPT_VDEV] );
        free( p_sys->psz_input_options );
        free( psz_tmp );
        free( psz_with_backslashes );
        return( VLC_ENOMEM );
    }

    sprintf( p_sys->psz_input_options[VLVC_IDX_OPT_ADEV],
             VLVC_OPT_ADEV_BASE, psz_with_backslashes );
    free( psz_tmp );
    free( psz_with_backslashes );

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: InitVideoSize
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             int: VLC return code:
 *                  VLC_SUCCESS in case of success
 *                  VLC_ENOMEM if not enough memory
 *
 * Description:
 *             Initializes the video size parameters specified by the user.
 *****************************************************************************/
#if defined( WIN32 )
int InitVideoSize( access_t *p_access, vlvc_param_t *p_sys )
{
    int            i_total_len;
    vlc_value_t    val;
    char           *psz_width;
    char           *psz_height;

    var_Create( p_access, VLVC_VAR_VIDEO_CAPTURE_SIZE, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_VIDEO_CAPTURE_SIZE, &val );

    psz_width = psz_video_capture_size_width_list[val.i_int];
    psz_height = psz_video_capture_size_height_list[val.i_int];

    i_total_len = strlen( psz_width ) + strlen( psz_height ) + strlen( VLVC_DSHOW_SIZE_BASE );
    p_sys->psz_input_options[VLVC_IDX_OPT_SIZE] =
        malloc( i_total_len  *
                sizeof( *p_sys->psz_input_options[VLVC_IDX_OPT_SIZE] ) );
    if( !p_sys->psz_input_options[VLVC_IDX_OPT_SIZE] )
    {
        free( p_sys->psz_input_options[VLVC_IDX_OPT_VDEV] );
        free( p_sys->psz_input_options[VLVC_IDX_OPT_ADEV] );
        free( p_sys->psz_input_options );
        return( VLC_ENOMEM );
    }
    snprintf( p_sys->psz_input_options[VLVC_IDX_OPT_SIZE], i_total_len,
              VLVC_DSHOW_SIZE_BASE, psz_width, psz_height );
    return( VLC_SUCCESS );
}
#else
int InitVideoSize( access_t *p_access, vlvc_param_t *p_sys )
{
    int            i_tmp;
    vlc_value_t    val;
    char           *psz_width;
    char           *psz_height;

    var_Create( p_access, VLVC_VAR_VIDEO_CAPTURE_SIZE, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_VIDEO_CAPTURE_SIZE, &val );

    psz_width = psz_video_capture_size_width_list[val.i_int];
    psz_height = psz_video_capture_size_height_list[val.i_int];

    i_tmp = strlen( psz_width ) + strlen( VLVC_V4L_WIDTH_BASE );
    p_sys->psz_input_options[VLVC_IDX_OPT_WIDTH] =
        malloc( i_tmp  *
                sizeof( *p_sys->psz_input_options[VLVC_IDX_OPT_WIDTH] ) );
    if( !p_sys->psz_input_options[VLVC_IDX_OPT_WIDTH] )
    {
        free( p_sys->psz_input_options[VLVC_IDX_OPT_VDEV] );
        free( p_sys->psz_input_options[VLVC_IDX_OPT_ADEV] );
        free( p_sys->psz_input_options );
        return( VLC_ENOMEM );
    }
    snprintf( p_sys->psz_input_options[VLVC_IDX_OPT_WIDTH], i_tmp,
              VLVC_V4L_WIDTH_BASE, psz_width );

    i_tmp = strlen( psz_height ) + strlen( VLVC_V4L_HEIGHT_BASE );
    p_sys->psz_input_options[VLVC_IDX_OPT_HEIGHT] =
        malloc( i_tmp  *
                sizeof( *p_sys->psz_input_options[VLVC_IDX_OPT_HEIGHT] ) );
    if( !p_sys->psz_input_options[VLVC_IDX_OPT_HEIGHT] )
    {
        free( p_sys->psz_input_options[VLVC_IDX_OPT_VDEV] );
        free( p_sys->psz_input_options[VLVC_IDX_OPT_WIDTH] );
        free( p_sys->psz_input_options );
        return( VLC_ENOMEM );
    }
    snprintf( p_sys->psz_input_options[VLVC_IDX_OPT_HEIGHT], i_tmp,
              VLVC_V4L_HEIGHT_BASE, psz_height );

    return( VLC_SUCCESS );
}
#endif

/*****************************************************************************
 * Name: GetOptions
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             int: VLC return code:
 *                  VLC_SUCCESS in case of success
 *                  VLC_ENOMEM if not enough memory
 *
 * Description:
 *             Initializes the VLM options.
 *****************************************************************************/
int GetOptions( access_t *p_access, vlvc_param_t *p_sys )
{
    int i_ret;

    p_sys->psz_input_options =
        malloc( VLVC_NB_INPUT_OPTION * sizeof( *p_sys->psz_input_options ) );
    if( !p_sys->psz_input_options )
        return( VLC_ENOMEM );

    if( ( i_ret = GetAudioVideoDevices( p_access, p_sys ) ) != VLC_SUCCESS )
        return( i_ret );

    if( ( i_ret = InitVideoSize( p_access, p_sys ) ) != VLC_SUCCESS )
        return( i_ret );

    p_sys->psz_input_options[VLVC_IDX_OPT_END] = NULL;

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: GetFixOverlay
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             int: VLC return code
 *
 * Description:
 *             Gets the boolean value if we have to disable the overlay
 *****************************************************************************/
int GetFixOverlay( access_t *p_access, vlvc_param_t *p_sys )
{
    vlc_value_t    val;

    var_Create( p_access, VLVC_VAR_FIX_OVERLAY, VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_FIX_OVERLAY, &val );
    p_sys->b_fix_overlay = val.b_bool;

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: GetMulticastMode
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             int: VLC return code
 *
 * Description:
 *             Gets the boolean value of the multicast checkbox.
 *****************************************************************************/
int GetMulticastMode( access_t *p_access, vlvc_param_t *p_sys )
{
    vlc_value_t    val;

    var_Create( p_access, VLVC_VAR_MULTICAST_MODE, VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
    var_Get( p_access, VLVC_VAR_MULTICAST_MODE, &val );
    p_sys->b_multicast = val.b_bool;

    return( VLC_SUCCESS );
}

/*****************************************************************************
 * Name: LaunchCoreModule
 *
 * Parameters:
 *             access_t * p_access: a pointer to the access modules structure
 *             vlvc_param_t *p_sys: the vlvc module specific structure
 *
 * Returns:
 *             void
 *
 * Description:
 *             Launches the core module.
 *****************************************************************************/
void LaunchCoreModule( vlc_object_t *p_this, vlvc_param_t *p_sys )
{
    vlvc_t *p_vlvc;

    p_vlvc = vlvc_New( p_this );
    vlvc_Init( p_vlvc, p_sys );
    if( p_vlvc->p_parameters->b_standalone == VLC_TRUE )
    {
        vlvc_SetCallbackChangeNbUsers( p_vlvc, NULL );
        vlvc_SetCallbackStop( p_vlvc, NULL );
        vlvc_SetCallbackCoreModuleMessage( p_vlvc, NULL );
        vlvc_SetCallbackChatMessage( p_vlvc, NULL );
        vlvc_Start( p_vlvc );
    }
    else
        vlvc_Delete( p_vlvc );
}
