/*****************************************************************************
 * vlvc_frame.hpp: VLC's graphical module headers for VLVC
 *****************************************************************************
 * Copyright (C) 2005 - 2008 the VLVC team
 * $Id: vlvc_frame.cpp
 *
 * Authors: Julien ARNAUD <arnaud_j@epitech.net>
 *          Julien CUPILLARD <cupill_j@epitech.net>
 *          Thomas DELON <delon_t@epitech.net>
 *          Arnaud DEREDEC <derede_a@epitech.net>
 *          Florian GUSTIN <gustin_f@epitech.net>
 *          Sylvestre HO TAM CHAY <ho-tam_s@epitech.net>
 *          Romain LAVOCAT <lavoca_r@epitech.net>
 *          William LIENARD <lienar_w@epitech.net>
 *          Guillaume MARTIN <martin_u@epitech.net>
 *          Florent NGUYEN <nguyen_c@epitech.net>
 *          Julien PASSERI <passer_j@epitech.net>
 *          Mathavarajan SUGUMARAN <suguma_m@epitech.net>
 *          David YU <yu_d@epitech.net>
 *
 * This module has been written in EPITECH <http://www.epitech.net>
 * EPITECH is computer science school in Paris - FRANCE -
 * under the direction of Flavien Astraud and Jerome Landrieu.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifndef _VLVCFRAME_H_
#define _VLVCFRAME_H_

#include <wx/datetime.h>
#include <wx/listctrl.h>
#include "wxwidgets.hpp"
#include "vlc_vlvc.h"

#define VLVC_USER_STATUS_ON             1
#define VLVC_USER_STATUS_OFF            0

#define VLVC_NO_USER_NAME               N_( "No name" )
#define VLVC_NO_USER_DESC               N_( "No description" )

#define VLVC_WINDOW_WIDTH               550
#define VLVC_WINDOW_HEIGHT              300

#define VLVC_CHAT_TEXTBOX_WIDTH         180
#define VLVC_CHAT_TEXTBOX_HEIGHT        100

#define VLVC_USER_LIST_WIDTH            138
#define VLVC_USER_LIST_HEIGHT           200

#define VLVC_SHOW_MESSAGE_EVENT         500
#define VLVC_ADD_USER_EVENT             ( VLVC_SHOW_MESSAGE_EVENT + 1 )
#define VLVC_DEL_USER_EVENT             ( VLVC_ADD_USER_EVENT + 1 )
#define VLVC_CHAT_MESSAGE_EVENT         ( VLVC_DEL_USER_EVENT + 1 )

/* Chat window defines */
#define VLVC_CHAT_TEXTBOX_SIZE          wxSize( VLVC_CHAT_TEXTBOX_WIDTH, VLVC_CHAT_TEXTBOX_HEIGHT )
#if defined( WIN32 )
# define VLVC_CHAT_TEXTBOX_SCROLL_LINES  1024
#else
# define VLVC_CHAT_TEXTBOX_SCROLL_LINES  0
#endif
#define END_OF_LINE                     "\n"
#define VLVC_TIMESTAMP_FORMAT           "[%H:%M] "
#define VLVC_CHAT_MESSAGE_JOIN          N_( " joined the conference" )
#define VLVC_CHAT_MESSAGE_LEAVE         N_( " left the conference" )
#define VLVC_AUTH_FORBID_MSG            N_( "You may not authorize yourself!" )
#define VLVC_UNAUTH_FORBID_MSG          N_( "You may not unauthorize yourself!" )
#define VLVC_SPRINTF_AUTH_ADMIN_MSG     N_( "You have authorized %s to speak." )
#define VLVC_SPRINTF_UNAUTH_ADMIN_MSG   N_( "You have unauthorized %s." )

/* User list defines */
#define VLVC_USER_LIST_SIZE             wxSize( VLVC_USER_LIST_WIDTH, VLVC_USER_LIST_HEIGHT )
#define VLVC_USER_LIST_FLAGS		( wxLC_REPORT | wxLC_SINGLE_SEL | wxLC_NO_HEADER )
#define VLVC_USER_LIST_COLUMN_WIDTH     134
#define VLVC_USER_LIST_POINT_SIZE       10

/* Message colors */
#define VLVC_CHAT_SYSTEM_MSG_COLOR      *wxBLUE
#define VLVC_CHAT_USER_MSG_COLOR        *wxBLACK
#define VLVC_CHAT_DESC_MSG_COLOR        *wxLIGHT_GREY
#define VLVC_CHAT_INFO_MSG_COLOR        wxColour( 0, 128, 0 )
#define VLVC_TEXT_BOX_DEFAULT_TEXT      wxU( _( NULL ) )

/* User list colors */
#define VLVC_USER_LIST_AUTH_COLOR       wxColour( 255, 102, 0 ) /* orange; vert = wxColour(51, 153, 0) */
#define VLVC_USER_LIST_UNAUTH_COLOR     *wxBLACK

/* User list font weight*/
#define VLVC_USER_LIST_AUTH_FONTWEIGHT          wxFONTWEIGHT_BOLD
#define VLVC_USER_LIST_UNAUTH_FONTWEIGHT        wxFONTWEIGHT_NORMAL

#define VLVC_BUTTON_TEXT_PLAY           wxU( _( "Play" ) )
#define VLVC_BUTTON_TEXT_STOP           wxU( _( "Stop" ) )
#define VLVC_BUTTON_TEXT_HAND           wxU( _( "Raise Hand" ) )
#define VLVC_BUTTON_TEXT_AUTH           wxU( _( "Authorize" ) )
#define VLVC_BUTTON_TEXT_UNAUTH         wxU( _( "Unauthorize" ) )
#define VLVC_BUTTON_TEXT_SEND           wxU( _( "Send" ) )
#define VLVC_FRAME_TEXT_USER            wxU( _( "Users" ) )
#define VLVC_FRAME_TEXT_INFOS           wxU( _( "Infos" ) )
#define VLVC_FRAME_TEXT_ACTION          wxU( _( "Actions" ) )
#define VLVC_FRAME_TEXT_CHAT            wxU( _( "Chat" ) )
#define VLVC_FRAME_TEXT_INFO_USER(user) wxU( "<" ) + (user) + wxU( "> " )

#define VLVC_WINDOW_NAME                "VLVC"

enum
{
    PressEnterEvent,
    SendEvent,
    PlayEvent,
    StopEvent,
    HandEvent,
    AuthEvent,
    UnauthEvent,
    UserList
    /* ComboSelect */
};

struct MessageEventData
{
    char psz_message[VLVC_MAX_MESSAGE_LEN];
    int  i_critical;
};

struct ChatMessageEventData
{
    char        psz_message[VLVC_MAX_SEND_TEXT_LEN];
    int         i_user_id;
    int         i_type;
};


namespace wxvlc
{
    struct s_user_info
    {
        int                    i_id;
        wxString               name;
        wxString               desc;
    };

    class VlvcFrame: public wxFrame
    {
    public :
        vlvc_t                  *p_vlvc;

        VlvcFrame( intf_thread_t *p_intf, wxWindow *p_parent);
        virtual ~VlvcFrame();

    protected:
        intf_thread_t           *p_intf;
        wxWindow                *p_parent;
        char                    p_tab_status[VLVC_MAX_USERS];
        s_user_info             tab_user_info[VLVC_MAX_USERS];
        bool                    b_first_added;

        /* Widgets */
        wxListView              *p_user_list;
        wxTextCtrl              *p_chat_textbox;
        wxTextCtrl              *p_entry_textbox;
        wxBitmapButton          *p_hand_button;
        wxBitmapButton          *p_auth_button;
        wxBitmapButton          *p_unauth_button;

        /* Widget callbacks functions */
        void                    OnButtonSendPress( wxCommandEvent& event );
        void                    OnButtonPlayPress( wxCommandEvent& event );
        void                    OnButtonStopPress( wxCommandEvent& event );
        void                    OnButtonHandPress( wxCommandEvent& event );
        void                    OnButtonAuthPress( wxCommandEvent& event );
        void                    OnButtonUnauthPress( wxCommandEvent& event );
        void                    OnUserListRightClick( wxListEvent& event );
        void                    OnCloseWindow( wxCloseEvent& event );

    public :
        void                    OnNbClientChange( int nb );
        void                    OnStopVlvc( );
        void                    OnCoreModuleMessage( char *psz_message, int i_critical );
        void                    OnIncomingChatMessage( int id_user, const char *psz_message,
                                                       int i_message_type );
        void                    DisableHandBtn( );
        void                    EnableAuthBtn( );
        void                    EnableUnauthBtn( );
        void                    OnClientAuth( int );
        void                    OnClientUnauth( int );

    private:
        void                    InitControls();
        wxPanel                 *GetMainPanel( wxFrame *parent );
        wxPanel                 *GetTopPanel( wxPanel *parent );
        wxPanel                 *GetChatPanel( wxPanel *parent );
        wxPanel                 *GetActionPanel( wxPanel *parent );
        wxPanel                 *GetUserPanel( wxPanel *parent );
        wxPanel                 *GetBtnPanel( wxPanel *parent );
        wxPanel                 *GetSendPanel( wxPanel *parent );

        void                    ChatMessage( wxString &message, const wxColour &color, bool bShowTimeStamp = true );

        void                    OnMessage( wxCommandEvent& );
        void                    OnAddUser( wxCommandEvent &evt );
        void                    OnDelUser( wxCommandEvent &evt );
        void                    OnChatMessage( wxCommandEvent &evt );
        void                    ShowMessage( char *psz_message, int i_critical );
        void                    AddUser( int );
        void                    DelUser( int );
        void                    ChangeColorUser( long, int );

        MessageEventData        messageData;
        ChatMessageEventData    chatMessageData;
        vlc_mutex_t             messageLock;
        int                     b_messageLocked;

        DECLARE_EVENT_TABLE();
    };
}

#endif // ! _VLVCFRAME_H_
